@@ -1,6 +1,7 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
+import os.path
 import json
 import logging
 
@@ -8,8 +9,8 @@ from math import ceil
 from requests.exceptions import HTTPError, ConnectionError
 from pypuppetdb.errors import EmptyResponseError
 
-from flask import abort
-
+from flask import abort, request, url_for
+from jinja2.utils import contextfunction
 
 # Python 3 compatibility
 try:
@@ -20,6 +21,21 @@ except NameError:
 log = logging.getLogger(__name__)
 
 
+@contextfunction
+def url_static_offline(context, value):
+    request_parts = os.path.split(os.path.dirname(context.name))
+    static_path = '/'.join(request_parts[1:])
+
+    return url_for('static', filename="%s/%s" % (static_path, value))
+
+
+def url_for_field(field, value):
+    args = request.view_args.copy()
+    args.update(request.args.copy())
+    args[field] = value
+    return url_for(request.endpoint, **args)
+
+
 def jsonprint(value):
     return json.dumps(value, indent=2, separators=(',', ': '))
 
