@@ -71,13 +71,6 @@ def mock_puppetdb_default_nodes(mocker):
              catalog_timestamp='2013-08-01T09:57:00.000Z',
              facts_timestamp='2013-08-01T09:57:00.000Z',
              status_report='unchanged'),
-        Node('_', 'node-skipped',
-             report_timestamp='2013-08-01T09:57:00.000Z',
-             latest_report_hash='1234567',
-             catalog_timestamp='2013-08-01T09:57:00.000Z',
-             facts_timestamp='2013-08-01T09:57:00.000Z',
-             status_report='skipped')
-
     ]
     return mocker.patch.object(app.puppetdb, 'nodes',
                                return_value=iter(node_list))
@@ -273,6 +266,9 @@ def test_offline_mode(client, mocker):
     assert soup.title.contents[0] == 'Puppetboard'
     for link in soup.find_all('link'):
         assert "//" not in link['href']
+        if 'offline' in link['href']:
+            static_rv = client.get(link['href'])
+            assert rv.status_code == 200
 
     for script in soup.find_all('script'):
         if "src" in script.attrs:
@@ -443,7 +439,6 @@ def test_radiator_view_json(client, mocker,
     assert json_data['noop'] == 1
     assert json_data['failed'] == 1
     assert json_data['changed'] == 1
-    assert json_data['skipped'] == 1
     assert json_data['unchanged'] == 1
 
 
@@ -537,9 +532,6 @@ def test_json_daily_reports_chart_ok(client, mocker):
         ]
     }
 
-    import logging
-    logging.error(query_data)
-
     dbquery = MockDbQuery(query_data)
 
     mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
@@ -558,3 +550,290 @@ def test_json_daily_reports_chart_ok(client, mocker):
         cur_day = next_day
 
     assert rv.status_code == 200
+
+
+def test_catalogs_disabled(client, mocker,
+                           mock_puppetdb_environments,
+                           mock_puppetdb_default_nodes):
+    app.app.config['ENABLE_CATALOG'] = False
+    rv = client.get('/catalogs')
+    assert rv.status_code == 403
+
+
+def test_catalogs_view(client, mocker,
+                       mock_puppetdb_environments,
+                       mock_puppetdb_default_nodes):
+    app.app.config['ENABLE_CATALOG'] = True
+    rv = client.get('/catalogs')
+    assert rv.status_code == 200
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+
+def test_catalogs_json(client, mocker,
+                       mock_puppetdb_environments,
+                       mock_puppetdb_default_nodes):
+    app.app.config['ENABLE_CATALOG'] = True
+    rv = client.get('/catalogs/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+    assert 'data' in result_json
+
+    for line in result_json['data']:
+        assert len(line) == 3
+        found_status = None
+        for status in ['failed', 'changed', 'unchanged', 'noop', 'unreported']:
+            val = BeautifulSoup(line[0], 'html.parser').find_all(
+                'a', {"href": "/node/node-%s" % status})
+            if len(val) == 1:
+                found_status = status
+                break
+        assert found_status, 'Line does not match any known status'
+
+        val = BeautifulSoup(line[2], 'html.parser').find_all(
+            'form', {"method": "GET",
+                     "action": "/catalogs/compare/node-%s" % found_status})
+        assert len(val) == 1
+
+
+def test_catalogs_json_compare(client, mocker,
+                               mock_puppetdb_environments,
+                               mock_puppetdb_default_nodes):
+    app.app.config['ENABLE_CATALOG'] = True
+    rv = client.get('/catalogs/compare/node-unreported/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+    assert 'data' in result_json
+
+    for line in result_json['data']:
+        assert len(line) == 3
+        found_status = None
+        for status in ['failed', 'changed', 'unchanged', 'noop', 'unreported']:
+            val = BeautifulSoup(line[0], 'html.parser').find_all(
+                'a', {"href": "/node/node-%s" % status})
+            if len(val) == 1:
+                found_status = status
+                break
+        assert found_status, 'Line does not match any known status'
+
+        val = BeautifulSoup(line[2], 'html.parser').find_all(
+            'form', {"method": "GET",
+                     "action": "/catalogs/compare/node-unreported...node-%s" %
+                     found_status})
+        assert len(val) == 1
+
+
+def test_facts_view(client, mocker, mock_puppetdb_environments):
+    query_data = {
+        'fact-names': [[chr(i) for i in range(ord('a'), ord('z') + 1)]]
+    }
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/facts')
+    assert rv.status_code == 200
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    searchable = soup.find('div', {'class': 'searchable'})
+    vals = searchable.find_all('div', {'class': 'column'})
+    assert len(vals) == 4
+
+
+def test_fact_view_with_graph(client, mocker,
+                              mock_puppetdb_environments,
+                              mock_puppetdb_default_nodes):
+    rv = client.get('/fact/architecture')
+    assert rv.status_code == 200
+
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    vals = soup.find_all('div', {"id": "factChart"})
+    assert len(vals) == 1
+
+
+def test_fact_view_without_graph(client, mocker,
+                                 mock_puppetdb_environments,
+                                 mock_puppetdb_default_nodes):
+    rv = client.get('/%2A/fact/augeas')
+    assert rv.status_code == 200
+
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    vals = soup.find_all('div', {"id": "factChart"})
+    assert len(vals) == 0
+
+
+def test_fact_value_view(client, mocker,
+                         mock_puppetdb_environments,
+                         mock_puppetdb_default_nodes):
+    rv = client.get('/fact/architecture/amd64')
+    assert rv.status_code == 200
+
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    vals = soup.find_all('div', {"id": "factChart"})
+    assert len(vals) == 0
+
+
+def test_node_view(client, mocker,
+                   mock_puppetdb_environments,
+                   mock_puppetdb_default_nodes):
+    rv = client.get('/node/node-failed')
+    assert rv.status_code == 200
+
+    soup = BeautifulSoup(rv.data, 'html.parser')
+    assert soup.title.contents[0] == 'Puppetboard'
+
+    vals = soup.find_all('table', {"id": "facts_table"})
+    assert len(vals) == 1
+
+    vals = soup.find_all('table', {"id": "reports_table"})
+    assert len(vals) == 1
+
+
+def test_fact_json_with_graph(client, mocker,
+                              mock_puppetdb_environments,
+                              mock_puppetdb_default_nodes):
+    values = ['a', 'b', 'b', 'd', True, 'a\nb']
+    query_data = {'facts': []}
+    query_data['facts'].append([])
+    for i, value in enumerate(values):
+        query_data['facts'][0].append({
+            'certname': 'node-%s' % i,
+            'name': 'architecture',
+            'value': value,
+            'environment': 'production'
+        })
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/fact/architecture/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+
+    assert 'data' in result_json
+    assert len(result_json['data']) == 6
+    for line in result_json['data']:
+        assert len(line) == 2
+
+    assert 'chart' in result_json
+    assert len(result_json['chart']) == 5
+    # Test group_by
+    assert result_json['chart'][0]['value'] == 2
+
+
+def test_fact_json_without_graph(client, mocker,
+                                 mock_puppetdb_environments,
+                                 mock_puppetdb_default_nodes):
+    values = ['a', 'b', 'b', 'd']
+    query_data = {'facts': []}
+    query_data['facts'].append([])
+    for i, value in enumerate(values):
+        query_data['facts'][0].append({
+            'certname': 'node-%s' % i,
+            'name': 'architecture',
+            'value': value,
+            'environment': 'production'
+        })
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/%2A/fact/augeas/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+
+    assert 'data' in result_json
+    assert len(result_json['data']) == 4
+    for line in result_json['data']:
+        assert len(line) == 2
+
+    assert 'chart' not in result_json
+
+
+def test_fact_value_json(client, mocker,
+                         mock_puppetdb_environments,
+                         mock_puppetdb_default_nodes):
+    values = ['a', 'b', 'b', 'd']
+    query_data = {'facts': []}
+    query_data['facts'].append([])
+    for i, value in enumerate(values):
+        query_data['facts'][0].append({
+            'certname': 'node-%s' % i,
+            'name': 'architecture',
+            'value': value,
+            'environment': 'production'
+        })
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/fact/architecture/amd64/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+
+    assert 'data' in result_json
+    assert len(result_json['data']) == 4
+    for line in result_json['data']:
+        assert len(line) == 1
+
+    assert 'chart' not in result_json
+
+
+def test_node_facts_json(client, mocker,
+                         mock_puppetdb_environments,
+                         mock_puppetdb_default_nodes):
+    values = ['a', 'b', 'b', 'd']
+    query_data = {'facts': []}
+    query_data['facts'].append([])
+    for i, value in enumerate(values):
+        query_data['facts'][0].append({
+            'certname': 'node-failed',
+            'name': 'fact-%s' % i,
+            'value': value,
+            'environment': 'production'
+        })
+
+    dbquery = MockDbQuery(query_data)
+
+    mocker.patch.object(app.puppetdb, '_query', side_effect=dbquery.get)
+
+    rv = client.get('/node/node-failed/facts/json')
+    assert rv.status_code == 200
+
+    result_json = json.loads(rv.data.decode('utf-8'))
+
+    assert 'data' in result_json
+    assert len(result_json['data']) == 4
+    for line in result_json['data']:
+        assert len(line) == 2
+
+    assert 'chart' not in result_json
+
+
+def test_offline_static(client):
+    rv = client.get('/offline/css/google_fonts.css')
+
+    assert 'Content-Type' in rv.headers
+    assert 'text/css' in rv.headers['Content-Type']
+    assert rv.status_code == 200
+
+    rv = client.get('/offline/Semantic-UI-2.1.8/semantic.min.css')
+    assert 'Content-Type' in rv.headers
+    assert 'text/css' in rv.headers['Content-Type']
+    assert rv.status_code == 200
